#define CONVERT_CYL_TO_RECT
/*****************************************************************************
*
* Copyright (c) 2000 - 2006, The Regents of the University of California
* Produced at the Lawrence Livermore National Laboratory
* All rights reserved.
*
* This file is part of VisIt. For details, see http://www.llnl.gov/visit/. The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or materials provided with the distribution.
*  - Neither the name of the UC/LLNL nor  the names of its contributors may be
*    used to  endorse or  promote products derived from  this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED.  IN  NO  EVENT  SHALL  THE  REGENTS  OF  THE  UNIVERSITY OF
* CALIFORNIA, THE U.S.  DEPARTMENT  OF  ENERGY OR CONTRIBUTORS BE  LIABLE  FOR
* ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//                            avtSplittingIncite4HDF5FileFormat.C                           //
// ************************************************************************* //

#include <avtSplittingIncite4HDF5FileFormat.h>

#include <string>
#include <list>

#include <vtkFloatArray.h>
#include <vtkRectilinearGrid.h>
#include <vtkStructuredGrid.h>
#include <vtkUnstructuredGrid.h>

#include <avtDatabaseMetaData.h>
#include <DebugStream.h>

#include <Expression.h>

#include <InvalidVariableException.h>

#ifdef PARALLEL
#include <avtParallel.h>
#endif

using     std::string;

namespace avtSplittingIncite4HDF5FileFormatAux {
  bool readDatasetDimensions(H5::H5File& h5File, int dims[3])
  {
    try {
      H5::Attribute cR = h5File.openGroup("/Coordinates").openAttribute("CoordinateRank");
      if (cR.getSpace().getSimpleExtentNdims() != 1) {
	std::cerr << "Error: Unexpected rank!" << std::endl;
	return false;
      }
      hsize_t nE;
      cR.getSpace().getSimpleExtentDims(&nE);
      if (nE != 3) {
	std::cerr << "Error: Unexpected number of dimensions!" << std::endl;
	return false;
      }
      cR.read(H5::PredType::NATIVE_INT, dims);
      return true;
    }
    catch (...) {
      return false;
    }

  }

  hsize_t numElementsInDataSet(H5::H5File& h5File, const char* dataSetName)
  {
      H5::DataSet dataset = h5File.openDataSet(dataSetName);
      H5::DataSpace dataspace = dataset.getSpace();

      hsize_t rank = dataspace.getSimpleExtentNdims();
      hsize_t dims[rank];
      dataspace.getSimpleExtentDims(dims);
      hsize_t numElementsInDataSet = 1;
      for (int i=0; i<rank; ++i) numElementsInDataSet *= dims[i];
      return numElementsInDataSet;
  }

  vtkFloatArray* readFloatDataSet(H5::H5File& h5File, const char* dataSetName, int expectedRank, hsize_t* dimsOut)
  {
    try {
      H5::DataSet dataset = h5File.openDataSet(dataSetName);
      H5::DataSpace dataspace = dataset.getSpace();

      if (dataspace.getSimpleExtentNdims() != expectedRank) {
	std::cerr << "Error: Unexpected rank!" << std::endl;
	return 0;
      }

      dataspace.getSimpleExtentDims(dimsOut);

      hsize_t numElementsInDataSet = 1;
      for (int i=0; i<expectedRank; ++i) numElementsInDataSet *= dimsOut[i];

      vtkFloatArray *data = vtkFloatArray::New();
      data->SetNumberOfTuples(numElementsInDataSet);
      float *dataArray = (float *)data->GetVoidPointer(0);
      dataset.read(dataArray, H5::PredType::NATIVE_FLOAT);

      return data;
    }
    catch (...)
    {
      return 0;
    }
  }

  vtkFloatArray* readPartOfFloatDataSet(H5::H5File& h5File, const char* dataSetName, hsize_t offset, hsize_t size)
  {
    try {
      H5::DataSet dataset = h5File.openDataSet(dataSetName);
      H5::DataSpace dataspace = dataset.getSpace();

      if (dataspace.getSimpleExtentNdims() != 1) {
	std::cerr << "Error: Unexpected rank!" << std::endl;
	return 0;
      }

      hsize_t numElementsInDataSet;
      dataspace.getSimpleExtentDims(&numElementsInDataSet);
      if (offset + size > numElementsInDataSet) 
	std::cerr << "offset + size > numElementsInDataSet" << std::endl;
      dataspace.selectHyperslab(H5S_SELECT_SET, &size, &offset);
      H5::DataSpace memSpace(1, &size);

      vtkFloatArray *data = vtkFloatArray::New();
      data->SetNumberOfTuples(size);
      float *dataArray = (float *)data->GetVoidPointer(0);
      dataset.read(dataArray, H5::PredType::NATIVE_FLOAT, memSpace, dataspace);

      return data;
    }
    catch (...)
    {
      return 0;
    }
  }
}



// ****************************************************************************
//  Method: avtSplittingIncite4HDF5 constructor
//
//  Programmer: ghweber -- generated by xml2avt
//  Creation:   Thu Feb 15 15:09:58 PST 2007
//
// ****************************************************************************

avtSplittingIncite4HDF5FileFormat::avtSplittingIncite4HDF5FileFormat(const char *filename)
    : avtSTSDFileFormat(filename), mFile(filename, H5F_ACC_RDONLY)
{
  H5::Exception::dontPrint();

  // Determine coordinate storage type
  try {
    mDims[0] = avtSplittingIncite4HDF5FileFormatAux::numElementsInDataSet(mFile, "/Coordinates/r");
    // If reading the r coordinate list data sets works, assume the file uses cylindrical coordinates
    mReadCylinderCoordinates = true;
  }
  catch (H5::GroupIException::GroupIException&)
  {
    // If reading the r coordinate list data set failed, assume the file uses rectilinear coordinates
    mReadCylinderCoordinates = false;
  }
  catch (...) {
    mDims[0] = mDims[1] = mDims[2] = 0;
    std::cerr << "Couldn't read dimensions." << std::endl;
  }

  if (mReadCylinderCoordinates) {
    // Determine the other two data set extents (theta and z)
    try {
      mDims[1] = avtSplittingIncite4HDF5FileFormatAux::numElementsInDataSet(mFile, "/Coordinates/theta");
      mDims[2] = avtSplittingIncite4HDF5FileFormatAux::numElementsInDataSet(mFile, "/Coordinates/z");
    }
    catch (...) {
      mDims[0] = mDims[1] = mDims[2] = 0;
      std::cerr << "Couldn't read dimensions." << std::endl;
    }
  }
  else {
    if (!avtSplittingIncite4HDF5FileFormatAux ::readDatasetDimensions(mFile, mDims)) {
      mDims[0] = mDims[1] = mDims[2] = 0;
      std::cerr << "Couldn't read dimensions." << std::endl;
    }
  }
}

// ****************************************************************************
//  Method: avtSplittingIncite4HDF5FileFormat::FreeUpResources
//
//  Purpose:
//      When VisIt is done focusing on a particular timestep, it asks that
//      timestep to free up any resources (memory, file descriptors) that
//      it has associated with it.  This method is the mechanism for doing
//      that.
//
//  Programmer: ghweber -- generated by xml2avt
//  Creation:   Thu Feb 15 15:09:58 PST 2007
//
// ****************************************************************************

void
avtSplittingIncite4HDF5FileFormat::FreeUpResources(void)
{
}


// ****************************************************************************
//  Method: avtSplittingIncite4HDF5FileFormat::PopulateDatabaseMetaData
//
//  Purpose:
//      This database meta-data object is like a table of contents for the
//      file.  By populating it, you are telling the rest of VisIt what
//      information it can request from you.
//
//  Programmer: ghweber -- generated by xml2avt
//  Creation:   Thu Feb 15 15:09:58 PST 2007
//
// ****************************************************************************

extern "C"  herr_t
add_var(hid_t loc_id, const char *varname, void *opData)
{
  std::list<std::string> *l = (std::list<std::string>*)opData;
  debug1 << "I: Adding variable " << varname << std::endl;
  l->push_back(varname);
  return 0;
}

void
avtSplittingIncite4HDF5FileFormat::PopulateDatabaseMetaData(avtDatabaseMetaData *md)
{
    //
    // CODE TO ADD A MESH
    //
    string meshname = "mesh";
    //
    // AVT_RECTILINEAR_MESH, AVT_CURVILINEAR_MESH, AVT_UNSTRUCTURED_MESH,
    // AVT_POINT_MESH, AVT_SURFACE_MESH, AVT_UNKNOWN_MESH
    avtMeshType mt;
    if (mReadCylinderCoordinates)
#ifdef CONVERT_CYL_TO_RECT
      mt = AVT_CURVILINEAR_MESH;
#else
      mt = AVT_RECTILINEAR_MESH;
#endif
    else
      mt = AVT_CURVILINEAR_MESH;
 
    int nblocks =1; // Single domain!!!
    md->SetFormatCanDoDomainDecomposition(true); // But we can split up the data if necessary
    int block_origin = 0;
    int spatial_dimension = 3;
    int topological_dimension = 3;
    double *extents = NULL;
    //
    // Here's the call that tells the meta-data object that we have a mesh:
    //
    AddMeshToMetaData(md, meshname, mt, extents, nblocks, block_origin,
                      spatial_dimension, topological_dimension);

    std::list<std::string> varList;
    debug1 << "Scanning file for variables." << std::endl;
    herr_t idx = H5Giterate(mFile.getId(), "/DataValues", 0, add_var, &varList);
    for (std::list<std::string>::iterator it = varList.begin(); it!= varList.end(); ++it)
      AddScalarVarToMetaData(md, *it, meshname, AVT_NODECENT);
    debug1 << "Done scanning file for variables." << std::endl;

    //
    // CODE TO ADD A VECTOR VARIABLE
    //
    // string mesh_for_this_var = meshname; // ??? -- could be multiple meshes
    // string varname = ...
    // int vector_dim = 2;
    //
    // AVT_NODECENT, AVT_ZONECENT, AVT_UNKNOWN_CENT
    // avtCentering cent = AVT_NODECENT;
    //
    //
    // Here's the call that tells the meta-data object that we have a var:
    //
    // AddVectorVarToMetaData(md, varname, mesh_for_this_var, cent,vector_dim);
    //

    //
    // CODE TO ADD A TENSOR VARIABLE
    //
    // string mesh_for_this_var = meshname; // ??? -- could be multiple meshes
    // string varname = ...
    // int tensor_dim = 9;
    //
    // AVT_NODECENT, AVT_ZONECENT, AVT_UNKNOWN_CENT
    // avtCentering cent = AVT_NODECENT;
    //
    //
    // Here's the call that tells the meta-data object that we have a var:
    //
    // AddTensorVarToMetaData(md, varname, mesh_for_this_var, cent,tensor_dim);
    //

    //
    // CODE TO ADD A MATERIAL
    //
    // string mesh_for_mat = meshname; // ??? -- could be multiple meshes
    // string matname = ...
    // int nmats = ...;
    // vector<string> mnames;
    // for (int i = 0 ; i < nmats ; i++)
    // {
    //     char str[32];
    //     sprintf(str, "mat%d", i);
    //     -- or -- 
    //     strcpy(str, "Aluminum");
    //     mnames.push_back(str);
    // }
    // 
    // Here's the call that tells the meta-data object that we have a mat:
    //
    // AddMaterialToMetaData(md, matname, mesh_for_mat, nmats, mnames);
    //
    //
    // Here's the way to add expressions:
    //Expression momentum_expr;
    //momentum_expr.SetName("momentum");
    //momentum_expr.SetDefinition("{u, v}");
    //momentum_expr.SetType(Expression::VectorMeshVar);
    //md->AddExpression(&momentum_expr);
    //Expression KineticEnergy_expr;
    //KineticEnergy_expr.SetName("KineticEnergy");
    //KineticEnergy_expr.SetDefinition("0.5*(momentum*momentum)/(rho*rho)");
    //KineticEnergy_expr.SetType(Expression::ScalarMeshVar);
    //md->AddExpression(&KineticEnergy_expr);
}


// ****************************************************************************
//  Method: avtSplittingIncite4HDF5FileFormat::GetMesh
//
//  Purpose:
//      Gets the mesh associated with this file.  The mesh is returned as a
//      derived type of vtkDataSet (ie vtkRectilinearGrid, vtkStructuredGrid,
//      vtkUnstructuredGrid, etc).
//
//  Arguments:
//      meshname    The name of the mesh of interest.  This can be ignored if
//                  there is only one mesh.
//
//  Programmer: ghweber -- generated by xml2avt
//  Creation:   Thu Feb 15 15:09:58 PST 2007
//
// ****************************************************************************

vtkDataSet *
avtSplittingIncite4HDF5FileFormat::GetMesh(const char *meshname)
{
  time_t before = time(0);
#ifdef PARALLEL
  debug1 << "I: In GetMesh() for processor " << PAR_Rank() << " of " << PAR_Size() << "." << std::endl;

  int numPartsForPartition = std::min(PAR_Size(), mDims[2]);
  if (PAR_Rank() > numPartsForPartition) {
    debug1 << "GetMesh(): No data left for processor. Returning 0." << std::endl;
    return 0;
  }
#else
  debug1 << "I: In serial GetMesh()." << std::endl;
#endif
  if (!mReadCylinderCoordinates) {
    debug1 << "xyz" << std::endl;
    hsize_t offset, size;
    offsetAndSizeForProcessor(offset, size);

    try {
      H5::DataSet xDataset = mFile.openDataSet("/Coordinates/x");
      H5::DataSet yDataset = mFile.openDataSet("/Coordinates/y");
      H5::DataSet zDataset = mFile.openDataSet("/Coordinates/z");

      H5::DataSpace xDataspace = xDataset.getSpace();
      H5::DataSpace yDataspace = yDataset.getSpace();
      H5::DataSpace zDataspace = zDataset.getSpace();

      if (xDataspace.getSimpleExtentNdims() != 1) {
	std::cerr << "Error: Unexpected rank for /Coordiantes/x!" << std::endl;
	return 0;
      }

      if (yDataspace.getSimpleExtentNdims() != 1) {
	std::cerr << "Error: Unexpected rank for /Coordiantes/y!" << std::endl;
	return 0;
      }

      if (zDataspace.getSimpleExtentNdims() != 1) {
	std::cerr << "Error: Unexpected rank for /Coordiantes/z!" << std::endl;
	return 0;
      }

      hsize_t xDim;
      hsize_t yDim;
      hsize_t zDim;

      xDataspace.getSimpleExtentDims(&xDim);
      yDataspace.getSimpleExtentDims(&yDim);
      zDataspace.getSimpleExtentDims(&zDim);

      if (xDim != yDim || xDim != zDim) {
	  std::cerr << "Error: Dimensions of coordinate data sets do not match!" << std::endl;
	  return 0;
      }
      if (offset + size > xDim) {
	std::cerr << "Error: Incorrect number of coordinates in data set." << std::endl;
	return 0;
      }

      float *xArray = new float[size];
      float *yArray = new float[size];
      float *zArray = new float[size];

      xDataspace.selectHyperslab(H5S_SELECT_SET, &size, &offset);
      yDataspace.selectHyperslab(H5S_SELECT_SET, &size, &offset);
      zDataspace.selectHyperslab(H5S_SELECT_SET, &size, &offset);

      H5::DataSpace memSpace(1, &size);
      xDataset.read(xArray, H5::PredType::NATIVE_FLOAT, memSpace, xDataspace);
      yDataset.read(yArray, H5::PredType::NATIVE_FLOAT, memSpace, yDataspace);
      zDataset.read(zArray, H5::PredType::NATIVE_FLOAT, memSpace, zDataspace);

      //
      // Create the vtkStructuredGrid and vtkPoints objects.
      //
      vtkStructuredGrid *sgrid  = vtkStructuredGrid::New();
      vtkPoints         *points = vtkPoints::New();
      sgrid->SetPoints(points);
      sgrid->SetDimensions(mDims[0], mDims[1], size / (mDims[0]*mDims[1]) );
      debug1 << "I: Dimensions of mesh are " << mDims[0] << " " << mDims[1] << " " << size / (mDims[0]*mDims[1]) << std::endl;
      points->Delete();
      points->SetNumberOfPoints(size);

      //
      // Copy the coordinate values into the vtkPoints object.
      //
      float *pts = (float *) points->GetVoidPointer(0);
      float *xc = xArray;
      float *yc = yArray;
      float *zc = zArray;

      for(int i = 0; i < size; ++i) {
	*pts++ = *xc++;
	*pts++ = *yc++;
	*pts++ = *zc++;
      }

      // Delete temporary arrays.
      delete [] xArray;
      delete [] yArray;
      delete [] zArray;

      time_t after = time(0);
      debug1 << "GetMesh() took " << after - before << " seconds." << std::endl;
      return sgrid;
    }
    catch (...)
    {
      return 0;
    }
  }
  else {
    debug1 << "rthetaz" << std::endl;
int firstSlice, lastSlice, numSlices;
    firstAndLastSliceForProcessor(firstSlice, lastSlice);
    numSlices = lastSlice - firstSlice + 1;


    hsize_t dummy;
    vtkFloatArray *coords[3] = {0,0,0};

    coords[0] = avtSplittingIncite4HDF5FileFormatAux::readFloatDataSet(mFile, "/Coordinates/r", 1, &dummy);
    coords[1] = avtSplittingIncite4HDF5FileFormatAux::readFloatDataSet(mFile, "/Coordinates/theta", 1, &dummy);
    coords[2] = avtSplittingIncite4HDF5FileFormatAux::readPartOfFloatDataSet(mFile, "/Coordinates/z", firstSlice, numSlices);

    #ifdef CONVERT_CYL_TO_RECT
    //
    // Create the vtkStructuredGrid and vtkPoints objects.
    //
    vtkStructuredGrid *sgrid  = vtkStructuredGrid::New();
    vtkPoints         *points = vtkPoints::New();
    sgrid->SetPoints(points);
    sgrid->SetDimensions(mDims[0], mDims[1], numSlices);
    points->Delete();
    points->SetNumberOfPoints(mDims[0] * mDims[1] * numSlices);

    //
    // Copy the coordinate values into the vtkPoints object.
    //
    float *pts = (float *) points->GetVoidPointer(0);

    for(int i = 0; i < mDims[0]; ++i) {
      for(int j = 0; j < mDims[1]; ++j) {
	float x = coords[0]->GetValue(i) * cos(coords[1]->GetValue(j));
	float y = coords[0]->GetValue(i) * sin(coords[1]->GetValue(j));
	for(int k = 0; k < numSlices; ++k) {
	  pts[((k*mDims[1]+j)*mDims[0]+i)*3+0] = x;
	  pts[((k*mDims[1]+j)*mDims[0]+i)*3+1] = y;
	  pts[((k*mDims[1]+j)*mDims[0]+i)*3+2] = coords[2]->GetValue(k);
	}
      }
    }
    coords[0]->Delete();
    coords[1]->Delete();
    coords[2]->Delete();
    time_t after = time(0);
    debug1 << "GetMesh() took " << after - before << " seconds." << std::endl;
    return sgrid;
#else
    //
    // Create the vtkRectilinearGrid object and set its dimensions
    // and coordinates.
    //
    vtkRectilinearGrid *rgrid = vtkRectilinearGrid::New();
    rgrid->SetDimensions(mDims[0], mDims[1], numSlices);
    rgrid->SetXCoordinates(coords[0]);
    coords[0]->Delete();
    rgrid->SetYCoordinates(coords[1]);
    coords[1]->Delete();
    rgrid->SetZCoordinates(coords[2]);
    coords[2]->Delete();
    time_t after = time(0);
    debug1 << "GetMesh() took " << after - before << " seconds." << std::endl;
    return rgrid;
#endif
  }
}

// ****************************************************************************
//  Method: avtSplittingIncite4HDF5FileFormat::GetVar
//
//  Purpose:
//      Gets a scalar variable associated with this file.  Although VTK has
//      support for many different types, the best bet is vtkFloatArray, since
//      that is supported everywhere through VisIt.
//
//  Arguments:
//      varname    The name of the variable requested.
//
//  Programmer: ghweber -- generated by xml2avt
//  Creation:   Thu Feb 15 15:09:58 PST 2007
//
// ****************************************************************************

vtkDataArray *
avtSplittingIncite4HDF5FileFormat::GetVar(const char *varname)
{
#ifdef PARALLEL
  int numPartsForPartition = std::min(PAR_Size(), mDims[2]);
  if (PAR_Rank() > numPartsForPartition) {
    debug1 << "GetMesh(): No data left for processor. Returning 0." << std::endl;
    return 0;
  }
#endif

  const char dsPrefix[] = "/DataValues/";
  char dsName[strlen(dsPrefix) + strlen(varname) + 1];
  strcpy(dsName, dsPrefix);
  strcat(dsName, varname);
  debug1 << "I: Reading values for variable " << varname << " from " << dsName << std::endl;
  hsize_t offset, size;

  offsetAndSizeForProcessor(offset, size);
  return  avtSplittingIncite4HDF5FileFormatAux::readPartOfFloatDataSet(mFile, dsName, offset, size);
    //
    // If you have a file format where variables don't apply (for example a
    // strictly polygonal format like the STL (Stereo Lithography) format,
    // then uncomment the code below.
    //
    // EXCEPTION1(InvalidVariableException, varname);
    //

    //
    // If you do have a scalar variable, here is some code that may be helpful.
    //
    // int ntuples = XXX; // this is the number of entries in the variable.
    // vtkFloatArray *rv = vtkFloatArray::New();
    // rv->SetNumberOfTuples(ntuples);
    // for (int i = 0 ; i < ntuples ; i++)
    // {
    //      rv->SetTuple1(i, VAL);  // you must determine value for ith entry.
    // }
    //
    // return rv;
    //
}


// ****************************************************************************
//  Method: avtSplittingIncite4HDF5FileFormat::GetVectorVar
//
//  Purpose:
//      Gets a vector variable associated with this file.  Although VTK has
//      support for many different types, the best bet is vtkFloatArray, since
//      that is supported everywhere through VisIt.
//
//  Arguments:
//      varname    The name of the variable requested.
//
//  Programmer: ghweber -- generated by xml2avt
//  Creation:   Thu Feb 15 15:09:58 PST 2007
//
// ****************************************************************************

vtkDataArray *
avtSplittingIncite4HDF5FileFormat::GetVectorVar(const char *varname)
{
    return 0;
    //
    // If you have a file format where variables don't apply (for example a
    // strictly polygonal format like the STL (Stereo Lithography) format,
    // then uncomment the code below.
    //
    // EXCEPTION1(InvalidVariableException, varname);
    //

    //
    // If you do have a vector variable, here is some code that may be helpful.
    //
    // int ncomps = YYY;  // This is the rank of the vector - typically 2 or 3.
    // int ntuples = XXX; // this is the number of entries in the variable.
    // vtkFloatArray *rv = vtkFloatArray::New();
    // int ucomps = (ncomps == 2 ? 3 : ncomps);
    // rv->SetNumberOfComponents(ucomps);
    // rv->SetNumberOfTuples(ntuples);
    // float *one_entry = new float[ucomps];
    // for (int i = 0 ; i < ntuples ; i++)
    // {
    //      int j;
    //      for (j = 0 ; j < ncomps ; j++)
    //           one_entry[j] = ...
    //      for (j = ncomps ; j < ucomps ; j++)
    //           one_entry[j] = 0.;
    //      rv->SetTuple(i, one_entry); 
    // }
    //
    // delete [] one_entry;
    // return rv;
    //
}

void avtSplittingIncite4HDF5FileFormat::firstAndLastSliceForProcessor(int& firstSlice, int& lastSlice)
{
#ifdef PARALLEL
  int numPartsForPartition = std::min(PAR_Size(), mDims[2]);
  float slicesPerProcessor = float(mDims[2]) / float(numPartsForPartition);
  debug1 << "Slices per processor: " << slicesPerProcessor << std::endl;

  firstSlice = int(PAR_Rank()*slicesPerProcessor);
  lastSlice = int((PAR_Rank()+1)*slicesPerProcessor);
  if (PAR_Rank() == numPartsForPartition - 1) {
    lastSlice = mDims[2] - 1; // Make sure that we include the final slice and do not include anything beyond
  }
  debug1 << "Processor: " << PAR_Rank() << " of " << PAR_Size() << " First slice: " << firstSlice<< " Last slice: " << lastSlice << std::endl;
#else
  firstSlice = 0;
  lastSlice = mDims[2] - 1;
  debug1 << "Serial mode. First slice: " << firstSlice<< " Last slice: " << lastSlice << std::endl;
#endif
}

void avtSplittingIncite4HDF5FileFormat::offsetAndSizeForProcessor(hsize_t& offset, hsize_t& size)
{
  int firstSlice, lastSlice;
  firstAndLastSliceForProcessor(firstSlice, lastSlice);

  offset = firstSlice * mDims[0] * mDims[1];
  size = (lastSlice - firstSlice + 1) * mDims[0] * mDims[1];
#ifdef PARALLEL
  debug1 << "Processor: " << PAR_Rank() << " of " << PAR_Size() << " Offset: " << offset << " Size: " << size << std::endl;
#else
  debug1 << " Offset: " << offset << " Size: " << size << std::endl;
#endif
}
